<?php
/**
 * @copyright	Copyright (C) 2009-2020 ACYBA SAS - All rights reserved.
 * @license		GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');

class plgSystemAcymailingtoolbox extends JPlugin
{
	public function __construct(&$subject, $config){
		parent::__construct($subject, $config);
		if(!isset($this->params)){
			include_once(rtrim(JPATH_ADMINISTRATOR,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_acymailing'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'helper.php');
			$plugin = JPluginHelper::getPlugin('system', 'acymailingtoolbox');
			$this->params = new acyParameter( $plugin->params );
		}
	}

	public function onAfterRender(){
		include_once(rtrim(JPATH_ADMINISTRATOR,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_acymailing'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'helper.php');
		$config = acymailing_config();
		if(version_compare($config->get('version'), '5.2.0', '<')){
			acymailing_display('Please download and install the latest AcyMailing version otherwise the Toolbox plugin will NOT work','error');
			return;
		}

		if(acymailing_isAdmin() || strpos($this->params->get('excluded', 'com_acysms'), acymailing_getVar('cmd', 'option')) !== false || (acymailing_getVar('cmd', 'option') == 'com_content' && acymailing_getVar('cmd', 'layout') == 'edit')) return;

        if (ACYMAILING_J40) {
            $body = JFactory::getApplication()->getBody(false);
        } else {
            $body = JResponse::getBody();
        }
		$match = '#(?:{|%7B)acymailing:(.*)(?:}|%7D)#Ui';
		$found = preg_match_all($match,$body,$results);
		if(!$found) return;

		//We got some results...
		foreach($results[0] as $i => $oneTag){
			if(isset($this->tags[$oneTag])) continue;

			$arguments = explode('|',strip_tags($results[1][$i]));
			//The first argument is a list of sections and cats...
			$parameter = new stdClass();
			$parameter->function = trim(strtolower($arguments[0]));
			for($i=1;$i<count($arguments);$i++){
				$args = explode(':',$arguments[$i]);
				$arg0 = trim($args[0]);
				if(isset($args[1])){
					$parameter->$arg0 = $args[1];
				}else{
					$parameter->$arg0 = true;
				}
			}

			$functionName = '_'.$parameter->function;
			if(method_exists($this,$functionName)){
				$this->tags[$oneTag] = $this->$functionName($parameter);
			}else{
				$this->tags[$oneTag] = 'Function not found : '.$functionName;
			}
		}

		$body = str_replace(array_keys($this->tags),$this->tags,$body);

        if (ACYMAILING_J40) {
            JFactory::getApplication()->setBody($body);
        } else {
            JResponse::setBody($body);
        }
	}

	/**
	 * Display the user information on a page
	 * example : {acymailing:userinfo|field:name}
	 * You can specify a default value in case of the user is not identified or the field not found
	 * example {acymailing:userinfo|field:name|default:Please log in first}
	 */
	private function _userinfo(&$parameter){
		if(empty($parameter->field)) return "Please specify a field, example : {acymailing:userinfo|field:name}";

		//We identify the user with his key/subid or if he's logged in
		$subscriberClass = acymailing_get('class.subscriber');
		$currentUser = $subscriberClass->identify(true);

		if(empty($currentUser)){
			return (isset($parameter->default)) ? $parameter->default : 'User not found or key not valid';
		}

		$myField = strip_tags($parameter->field);
		if(!isset($currentUser->$myField)){
			return (isset($parameter->default)) ? $parameter->default : 'Field not found : '.$myField;
		}

		return $currentUser->$myField;
	}

	/**
	 * {acymailing:users} to display a list of users
	 * Extra parameters...
	 * fields:name,email to display the fields name and email
	 * order:sub.created ASC to order users by created date ASC
	 * limit:30 to limit the number of users displayed to 30 only
	 *
	 */
	private function _users(&$parameter){
		$selectedFields = empty($parameter->fields) ? $this->params->get('selectedfields','name') : $parameter->fields;
		$limit = empty($parameter->limit) ? $this->params->get('limit','50') : intval($parameter->limit);
		$order = empty($parameter->order) ? $this->params->get('order','sub.subid DESC') : $parameter->order;

		$selectedFieldsArray = explode(',',$selectedFields);
		$selectedFields = 'sub.`'.implode('`, sub.`',$selectedFieldsArray).'`';

		$gravatarFields = acymailing_loadResultArray('SELECT namekey FROM #__acymailing_fields WHERE type = "gravatar"');
		$replaceValues = array();
		foreach($gravatarFields as &$oneGravatar){
			if(in_array($oneGravatar, $selectedFieldsArray)) $replaceValues['sub.`'.$oneGravatar.'`'] = 'CONCAT(sub.`email`,"|",sub.`'.$oneGravatar.'`) AS '.$oneGravatar;
			else unset($oneGravatar);
		}

		if(!empty($replaceValues)){
			$selectedFields = str_replace(array_keys($replaceValues), $replaceValues, $selectedFields);
		}

		$filters = array();
		if(!empty($parameter->where)){
			$conditions = explode(';', str_replace(array('&lt;','&gt;'), array('<','>'), $parameter->where));
			foreach($conditions as $oneCondition){
				$values = explode(',', $oneCondition);
				if(count($values) != 3) continue;
				if(!in_array($values[1], array('=','!=','LIKE','<','>','<=','>='))) continue;
				$values[2] = str_replace(array('[day]','[month]','[year]'), array(date('d'), date('m'), date('Y')), $values[2]);
				if($values[1] == 'LIKE') $values[2] = '%'.$values[2].'%';
				$filters[] = 'sub.`'.acymailing_secureField($values[0]).'` '.$values[1].' '.acymailing_escapeDB($values[2]);
			}
		}

		if(!empty($parameter->distinct)) $selectedFields = 'DISTINCT '.$selectedFields;

		$query = 'SELECT '.$selectedFields.' FROM #__acymailing_subscriber AS sub';
		$filters[] = 'sub.accept = 1';
		if(!empty($parameter->listid)){
			$listids = explode(',', $parameter->listid);
			acymailing_arrayToInteger($listids);
			$query .= ' JOIN #__acymailing_listsub AS listsub ON listsub.subid = sub.subid';
			$filters[] = 'listsub.listid IN ('.implode(',', $listids).') AND listsub.status > 0';
		}
		if(!empty($parameter->confirmed)) $filters[] = 'sub.confirmed = 1';
		if(!empty($filters)) $query .= ' WHERE ('.implode(') AND (', $filters).')';

		$query .= ' ORDER BY '.$order.' LIMIT '.$limit;

		$users = acymailing_loadObjectList($query);

		if($users == null) return acymailing_getDBError();
		if(empty($users)) return;

		$return = '<table class="acyusers">';
		if(!empty($parameter->header)){
			$oneUser = reset($users);
			$return .= '<tr>';
			foreach($oneUser as $fieldName => $oneVal){
				$width = '';
				if(!empty($parameter->width)) $width = 'width="'.$parameter->width.'"';
				$propwidth = 'width_'.$fieldName;
				if(!empty($parameter->$propwidth)) $width = 'width="'.$parameter->$propwidth.'"';
				$proplabel = 'label_'.$fieldName;
				$return .= '<th class="acytd_'.$fieldName.'" '.$width.' >'.(empty($parameter->$proplabel) ? $fieldName : $parameter->$proplabel).'</th>';
			}
			$return .= '</tr>';
		}

		foreach($users as $oneUser){
			$return .= '<tr>';
			foreach($oneUser as $fieldName => $oneVal){
				if($fieldName == 'email' && !empty($parameter->mailto)) $oneVal = '<a href="mailto:'.$oneVal.'">'.$oneVal.'</a>';
				if(in_array($fieldName, $gravatarFields)){
					$userValues = explode('|', $oneVal);
					if(empty($userValues[1])) $oneVal = '<img alt="" src="https://www.gravatar.com/avatar/'.md5(strtolower(trim($userValues[0]))).'?d=mm&s=50" />';
					else $oneVal = '<img alt="" style="width:50px;height:50px;" src="'.ACYMAILING_LIVE.str_replace(DS,'/',trim(acymailing_cleanPath(html_entity_decode(acymailing_getFilesFolder())),DS.' ').DS.'userfiles/').$userValues[1].'" />';
				}
				$width = '';
				if(!empty($parameter->width)) $width = 'width="'.$parameter->width.'"';
				$propwidth = 'width_'.$fieldName;
				if(!empty($parameter->$propwidth)) $width = 'width="'.$parameter->$propwidth.'"';
				$return .= '<td class="acytd_'.$fieldName.'" '.$width.' >'.$oneVal.'</td>';
			}
			$return .= '</tr>';
		}
		$return .= '</table>';

		return $return;
	}

	private function _mail(&$parameter){
		$mailid = acymailing_getVar('int', 'mailid');
		$mailClass= acymailing_get('class.mail');
		$newsletter = $mailClass->get($mailid);
		if(empty($newsletter)) return "Newsletter not found";
		$myField = strip_tags($parameter->field);
		$result = $newsletter->$myField;
		if($myField == 'altbody') $result = nl2br($result);
		return $result;
	}

	private function _count(&$parameter){
		$query = 'SELECT COUNT(sub.subid) FROM #__acymailing_subscriber AS sub';

		$filters = array();
		if(!empty($parameter->where)){
			$conditions = explode(';', str_replace(array('&lt;','&gt;'), array('<','>'), $parameter->where));
			foreach($conditions as $oneCondition){
				$values = explode(',', $oneCondition);
				if(count($values) != 3) continue;
				if(!in_array($values[1], array('=','!=','LIKE','<','>','<=','>='))) continue;
				if($values[1] == 'LIKE') $values[2] = '%'.$values[2].'%';
				$filters[] = 'sub.`'.acymailing_secureField($values[0]).'` '.$values[1].' '.acymailing_escapeDB($values[2]);
			}
		}

		if(!empty($parameter->listid)){
			$query .= ' JOIN #__acymailing_listsub AS listsub ON sub.subid = listsub.subid';
			$filters[] = 'listsub.status > 0 AND listsub.listid = '.intval($parameter->listid);
		}

		if(!empty($filters)) $query .= ' WHERE ('.implode(') AND (', $filters).')';

		$count = acymailing_loadResult($query);

		$numberformat = $this->params->get('numberformat','space');
		$allNumbersFormat = array('space'=>' ','comma'=>',','nothing'=>'');
		$count = number_format($count, 0, '', $allNumbersFormat[$numberformat]);

		return $count;
	}

	function onContentPrepare($context, &$article, &$params, $limitstart = 0){
		return $this->onPrepareContent($article, $params, $limitstart);
	}

	function onPrepareContent(&$article, &$params, $limitstart = 0){
		$match = '#{acy:(.*)}#Ui';
		if(!preg_match_all($match,$article->text,$results)) return;

		$replaceResults = array();
		if(!include_once(rtrim(JPATH_ADMINISTRATOR,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_acymailing'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'helper.php')) return;

		foreach($results[0] as $i => $oneTag){
			$arguments = explode('|',strip_tags($results[1][$i]));
			$tag = new stdClass();
			$tag->mailid = (int) $arguments[0];
			for($i=1,$a=count($arguments);$i<$a;$i++){
				$args = explode(':',$arguments[$i]);
				if(isset($args[1])){
					$tag->{$args[0]} = $args[1];
				}else{
					$tag->{$args[0]} = true;
				}
			}

			if(empty($tag->mailid)){
				//Get the latest one which is published and visible
				$query = 'SELECT m.`mailid` FROM `#__acymailing_listmail` as lm LEFT JOIN `#__acymailing_mail` as m on lm.mailid = m.mailid ';
				$where = array();
				$where[] = 'm.`visible`= 1 AND m.`published` = 1';
				if(!empty($tag->list)){
					$where[] = 'lm.`listid` = '.(int) $tag->list;
				}else{
					$query .= ' LEFT JOIN `#__acymailing_list` as l on lm.listid = l.listid ';
					$where[] = 'l.`visible`= 1 AND l.`published` = 1';
				}
				$query .= ' WHERE ('.implode(') AND (',$where).')';
				$query .= ' ORDER BY m.`senddate` DESC, m.`mailid` DESC LIMIT 1';
				$tag->mailid = acymailing_loadResult($query);

				if(empty($tag->mailid)){
					$replaceResults[$oneTag] = 'Newsletter not found';
					continue;
				}
			}

			//We have a mailid now
			$mailClass = acymailing_get('helper.mailer');
			$oneMail = $mailClass->load($tag->mailid);
			if(empty($oneMail->mailid)){
				$replaceResults[$oneTag] = 'Newsletter not found: '.$tag->mailid;
				continue;
			}

			if(preg_match('#'.$oneTag.'#i', $oneMail->body))
			{
				$replaceResults[$oneTag] = 'This article was inserted in the newsletter';
				continue;
			}

			$currentEmail = acymailing_currentUserEmail();
			if(!empty($currentEmail)){
				$userClass = acymailing_get('class.subscriber');
				$receiver = $userClass->get($currentEmail);
			}

			if(empty($receiver)){
				$receiver = new stdClass();
				$receiver->name = acymailing_translation('VISITOR');
			}

			$oneMail->sendHTML = true;
			acymailing_trigger('acymailing_replaceusertags', array(&$oneMail, &$receiver, false));

			$replaceResults[$oneTag] = $oneMail->body;
		}

		$article->text = str_replace(array_keys($replaceResults),$replaceResults,$article->text);
	}
}//endclass
